# Ω*
#               ■          ■■■■■  
#               ■         ■■   ■■ 
#               ■        ■■     ■ 
#               ■        ■■       
#     ■■■■■     ■        ■■■      
#    ■■   ■■    ■         ■■■     
#   ■■     ■■   ■          ■■■■   
#   ■■     ■■   ■            ■■■■ 
#   ■■■■■■■■■   ■              ■■■
#   ■■          ■               ■■
#   ■■          ■               ■■
#   ■■     ■    ■        ■■     ■■
#    ■■   ■■    ■   ■■■  ■■■   ■■ 
#     ■■■■■     ■   ■■■    ■■■■■


"""
Moca System Core Module
モカシステムコアモジュール
茉客系统核心模块

[MocaCore]

Copyright (c) 2020.1.17 [el.ideal-ideas]

This software is released under the MIT License. see LICENSE.txt.

https://www.el-ideal-ideas.com
"""

# -- Imports --------------------------------------------------------------------------

from pathlib import Path
from typing import List, Union, Optional
from json import dump
from .utils import to_string_list, create_path_if_not_exists

# -------------------------------------------------------------------------- Imports --

# -- Public Functions --------------------------------------------------------------------------


def get_children_list(path: Union[Path, str]) -> List[Path]:
    """
    get children list
    :param path: target directory
    :return: children list
    """
    # set target directory
    target: Path
    if isinstance(path, str):
        target = Path(path)
    else:
        target = path
    if target.is_dir():  # check target directory
        return list(target.iterdir())
    else:
        return []


def get_children_list_as_str(path: Union[Path, str],
                             only_name: bool = False) -> List[str]:
    """
    get children list.
    :param path: target directory.
    :param only_name: only return item name without path.
    :return: children list
    """
    if only_name:
        return list(map(lambda v: v.name, get_children_list(path)))
    else:
        return to_string_list(get_children_list(path))


def get_children_file_list(path: Union[Path, str]) -> List[Path]:
    """
    get children file list.
    :param path: target directory.
    :return: children file list.
    """
    return [item for item in get_children_list(path) if item.is_file()]


def get_children_file_list_as_str(path: Union[Path, str],
                                  only_name: bool = False) -> List[str]:
    """
    get children file list.
    :param path: target directory.
    :param only_name: only return item name without path.
    :return: children file list.
    """
    if only_name:
        return list(map(lambda v: v.name, get_children_file_list(path)))
    else:
        return to_string_list(get_children_file_list(path))


def get_children_directory_list(path: Union[Path, str]) -> List[Path]:
    """
    get children directory list.
    :param path: target directory.
    :return: children directory list.
    """
    return [item for item in get_children_list(path) if item.is_dir()]


def get_children_directory_list_as_str(path: Union[Path, str],
                                       only_name: bool = False) -> List[str]:
    """
    get children directory list.
    :param path: target directory.
    :param only_name: only return item name without path.
    :return: children directory list.
    """
    if only_name:
        return list(map(lambda v: v.name, get_children_directory_list(path)))
    else:
        return to_string_list(get_children_directory_list(path))


def get_file_system_tree(path: Union[Path, str],
                         limit: int = 500) -> Optional[dict]:
    """
    get file system tree
    :param path: target directory
    :param limit: maximum number can scan directories
    :return: file system tree
    """
    # check limit
    if limit < 0:
        return None
    # set target directory
    target: Path
    if isinstance(path, str):
        target = Path(path)
    else:
        target = path
    # check target directory
    if not target.is_dir():
        return None
    tree = {
        '.__self_name__': target.name,
        '.__file_count__': 0,
        '.__directory__count__': 0
    }
    file_count = 0
    file_prefix = 'file_'
    directory_count = 0
    directory_prefix = 'directory_'
    for item in get_children_list(target):
        try:
            if item.is_dir():
                tree[directory_prefix + str(directory_count)] = get_file_system_tree(item, limit - 1)
                tree['.__directory__count__'] += 1
                directory_count += 1
            else:
                tree[file_prefix + str(file_count)] = item.name
                tree['.__file_count__'] += 1
                file_count += 1
        except (FileNotFoundError, PermissionError, OSError, Exception) as error:
            tree[file_prefix + str(file_count)] = f"Can't access to this item <Exception: {error}>"
            tree['.__file_count__'] += 1
            file_count += 1
    return tree


def dump_file_system_tree(target_path: Union[Path, str],
                          output: Union[Path, str],
                          limit: int = 500) -> None:
    """
    dump system file tree as json to file.
    :param target_path: target directory.
    :param output: dumped file name.
    :param limit: maximum number can scan files.
    :return: status, [success] or [failed]
    """
    create_path_if_not_exists(output)
    with open(str(output), mode='w', encoding='utf-8') as output_file:
        dump(get_file_system_tree(target_path, limit), output_file,
             ensure_ascii=False, indent=4, sort_keys=True, separators=(',', ': '))

# -------------------------------------------------------------------------- Public Functions --
