# Ω*
#               ■          ■■■■■  
#               ■         ■■   ■■ 
#               ■        ■■     ■ 
#               ■        ■■       
#     ■■■■■     ■        ■■■      
#    ■■   ■■    ■         ■■■     
#   ■■     ■■   ■          ■■■■   
#   ■■     ■■   ■            ■■■■ 
#   ■■■■■■■■■   ■              ■■■
#   ■■          ■               ■■
#   ■■          ■               ■■
#   ■■     ■    ■        ■■     ■■
#    ■■   ■■    ■   ■■■  ■■■   ■■ 
#     ■■■■■     ■   ■■■    ■■■■■


"""
Moca System Core Module
モカシステムコアモジュール
茉客系统核心模块

[MocaCore]

Copyright (c) 2020.1.17 [el.ideal-ideas]

This software is released under the MIT License. see LICENSE.txt.

https://www.el-ideal-ideas.com
"""


# -- Imports --------------------------------------------------------------------------

import psutil
from typing import List

# -------------------------------------------------------------------------- Imports --

# -- Public Functions --------------------------------------------------------------------------


def get_process_id() -> int:
    """Return the process id"""
    process = psutil.Process()
    return int(process.pid)


def get_process_name() -> str:
    """Return the process name"""
    process = psutil.Process()
    return str(process.name())


def get_process_username() -> str:
    """Return the process username"""
    process = psutil.Process()
    return str(process.username())


def get_process_cpu_percent() -> float:
    """Return the process cpu percentage"""
    process = psutil.Process()
    return float(process.cpu_percent(interval=1))


def get_process_memory_percent() -> float:
    """Return the process memory percentage"""
    process = psutil.Process()
    return float(process.memory_percent())


def get_process_memory_used() -> int:
    """Return the process memory used bytes"""
    process = psutil.Process()
    return int(process.memory_info().rss)


def get_process_thread_count() -> int:
    """Return the process thread count"""
    process = psutil.Process()
    return int(process.num_threads())


def get_process_info() -> dict:
    """Return the process information"""
    process = psutil.Process()
    info = {
        'pid': int(process.pid),
        'name': str(process.name()),
        'user_name': str(process.username()),
        'cpu_percent': float(process.cpu_percent(interval=1)),
        'memory_used': int(process.memory_info().rss),
        'memory_percent': float(process.memory_percent()),
        'thread_count': int(process.num_threads()),
    }
    return info


def get_process_environment() -> dict:
    """Return process environment"""
    process = psutil.Process()
    return process.environ()


def get_cpu_percent() -> List[float]:
    """Return cpu percentage"""
    scpupercent_per_cpu = psutil.cpu_percent(interval=1, percpu=True)
    return scpupercent_per_cpu


def get_cpu_count(logical=True) -> int:
    """Return the number of cpu cores"""
    return psutil.cpu_count(logical=logical)


def get_cpu_info() -> dict:
    """Return cpu information"""
    scputimes = psutil.cpu_times()
    scpustats = psutil.cpu_stats()
    scpufrep = psutil.cpu_freq()
    scpupercent_per_cpu = psutil.cpu_percent(interval=1, percpu=True)
    scpupercent = psutil.cpu_percent(interval=1, percpu=False)
    info = {
        'cpu_count(logical)': psutil.cpu_count(),
        'cpu_count': psutil.cpu_count(logical=False),
        'cpu_percent(per_cpu)': scpupercent_per_cpu,
        'cpu_percent': scpupercent,
        'cpu_status': {
            'ctx_switches': scpustats.ctx_switches,
            'interrupts': scpustats.interrupts,
            'soft_interrupts': scpustats.soft_interrupts,
            'sys_calls': scpustats.syscalls
        },
        'cpu_freq': {
            'current': scpufrep.current,
            'min': scpufrep.min,
            'max': scpufrep.max,
        },
        'cpu_times': {
            'user': scputimes.user,
            'nice': scputimes.nice,
            'system': scputimes.system,
            'idle': scputimes.idle,
        },
    }
    return info


def get_memory_info() -> dict:
    """Return memory information"""
    process = psutil.Process()
    pmem = process.memory_info()
    svmem = psutil.virtual_memory()
    info = {
        'total': svmem.total,
        'available': svmem.available,
        'used': svmem.used,
        'free': svmem.free,
        'active': svmem.active,
        'inactive': svmem.inactive,
        'percent': svmem.percent,
        'self_used': {
            'percent': process.memory_percent(),
            'rss': pmem.rss,
            'vms': pmem.vms,
        }
    }
    return info


def get_swap_info() -> dict:
    """Return swap information"""
    sswap = psutil.swap_memory()
    info = {
        'total': sswap.total,
        'used': sswap.used,
        'free': sswap.free,
        'percent': sswap.percent,
    }
    return info


def get_disk_info() -> dict:
    """Return disk information"""
    sdiskpart = psutil.disk_partitions()
    sdiskio = psutil.disk_io_counters()
    info = {
        'partitions': [{'device': item.device,
                        'mountpoint': item.mountpoint,
                        'fstype': item.fstype,
                        'opts': item.opts}
                       for item in sdiskpart],
        'usage': {mount: {'total': psutil.disk_usage(mount).total,
                          'used': psutil.disk_usage(mount).used,
                          'free': psutil.disk_usage(mount).free,
                          'percent': psutil.disk_usage(mount).percent}
                  for mount in [item.mountpoint for item in sdiskpart]},
        'sdiskio': {
            'read_count': sdiskio.read_count,
            'write_count': sdiskio.write_count,
            'read_bytes': sdiskio.read_bytes,
            'write_bytes': sdiskio.write_bytes,
            'read_time': sdiskio.read_time,
            'write_time': sdiskio.write_time,
        }
    }
    return info

# -------------------------------------------------------------------------- Public Functions --
